const path = require('path');
const moduleToCdn = require('@talend/module-to-cdn');
const DynamicCdnWebpackPlugin = require('dynamic-cdn-webpack-plugin');

function init()  {
    moduleToCdn.add({
        '@talend/react-components': {
            var: 'TalendReactComponents',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactComponents.js',
                    production: '/dist/TalendReactComponents.min.js',
                }
            }
        },
        '@talend/react-cmf': {
            var: 'TalendReactCmf',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactCmf.js',
                    production: '/dist/TalendReactCmf.min.js',
                }
            }
        },
        '@talend/react-containers': {
            var: 'TalendReactContainers',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactContainers.js',
                    production: '/dist/TalendReactContainers.min.js',
                }
            }
        },
        '@talend/react-datagrid': {
            var: 'TalendReactDatagrid',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactDatagrid.js',
                    production: '/dist/TalendReactDatagrid.min.js',
                }
            }
        }
    });
}

function getCopyConfig() {
    const config = [];
    // TODO: support yarn.lock file and move all that code in the webpack dynmaic cdn;
    const packagelock = require(path.join(process.cwd(), 'package-lock.json'));
    Object.keys(packagelock.dependencies).forEach(moduleName => {
        const info = moduleToCdn(moduleName, packagelock.dependencies[moduleName].version, { env: 'development' });
        if (info && info.local) {
            config.push({ from: info.local, to: `cdn/${info.name}/${info.version}${info.path}`});
        }
    });
	return config;
}

function resolver(...args) {
    // will change once we have publish our CDN
	return moduleToCdn(...args);
}

function localResolver(...args) {
    const info = moduleToCdn(...args);
    if (!info) {
        return info;
    }
    info.url = `/cdn/${info.name}/${info.version}${info.path}`;
	return info;
}

function getWebpackPlugin(isEnvCDN) {
    return new DynamicCdnWebpackPlugin({ resolver: isEnvCDN ? resolver : localResolver });
}

module.exports = {
    resolver,
    localResolver,
    init,
    getCopyConfig,
    getWebpackPlugin,
};